/* slurp-resource.c
 *
 * Copyright © 2017 Julian Richen <julian@richen.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define G_LOG_DOMAIN "slurp-resource"

#include "slurp-resource.h"

G_DEFINE_BOXED_TYPE (SlurpResource, slurp_resource, slurp_resource_ref, slurp_resource_unref)

SlurpResource *
slurp_resource_new (void)
{
    SlurpResource *self;

    self = g_slice_new0 (SlurpResource);
    self->ref_count = 1;

    self->uri = NULL;
    self->destination = NULL;
    self->overwrite = FALSE;
    self->file = NULL;

    //self->buffer[16384] = {0};
    self->output = NULL;

    self->total_bytes = 0;
    self->downloaded_bytes = 0;
    self->last_progress_time = 0;

    return self;
}

SlurpResource *
slurp_resource_copy (SlurpResource *self)
{
    SlurpResource *copy;

    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    copy = slurp_resource_new ();

    return copy;
}

static void
slurp_resource_free (SlurpResource *self)
{
    g_assert (self);
    g_assert_cmpint (self->ref_count, ==, 0);

    g_clear_pointer (&self->uri, g_free);
    g_clear_pointer (&self->destination, g_free);
    g_clear_pointer (&self->file, g_object_unref);
    g_clear_pointer (&self->output, g_object_unref);

    if (self)
        g_slice_free (SlurpResource, self);
}

SlurpResource *
slurp_resource_ref (SlurpResource *self)
{
    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    g_atomic_int_inc (&self->ref_count);

    return self;
}

void
slurp_resource_unref (SlurpResource *self)
{
    g_return_if_fail (self);
    g_return_if_fail (self->ref_count);

    if (g_atomic_int_dec_and_test (&self->ref_count))
        slurp_resource_free (self);
}
