/* slurp-manager-job.c
 *
 * Copyright © 2018 Julian Richen <julian@richen.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "slurp-manager-job.h"

G_DEFINE_BOXED_TYPE (SlurpManagerJob, slurp_manager_job, slurp_manager_job_ref, slurp_manager_job_unref)

SlurpManagerJob *
slurp_manager_job_new (void)
{
    SlurpManagerJob *self;

    self = g_slice_new0 (SlurpManagerJob);
    self->ref_count = 1;

    self->resource = NULL;
    self->cancellable = NULL;

    return self;
}

SlurpManagerJob *
slurp_manager_job_copy (SlurpManagerJob *self)
{
    SlurpManagerJob *copy;

    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    copy = slurp_manager_job_new ();

    return copy;
}

static void
slurp_manager_job_free (SlurpManagerJob *self)
{
    g_assert (self);
    g_assert_cmpint (self->ref_count, ==, 0);

    g_clear_pointer (&self->resource, g_object_unref);
    g_clear_pointer (&self->cancellable, g_object_unref);

    g_slice_free (SlurpManagerJob, self);
}

SlurpManagerJob *
slurp_manager_job_ref (SlurpManagerJob *self)
{
    g_return_val_if_fail (self, NULL);
    g_return_val_if_fail (self->ref_count, NULL);

    g_atomic_int_inc (&self->ref_count);

    return self;
}

void
slurp_manager_job_unref (SlurpManagerJob *self)
{
    g_return_if_fail (self);
    g_return_if_fail (self->ref_count);

    if (g_atomic_int_dec_and_test (&self->ref_count))
        slurp_manager_job_free (self);
}
