/* slurp-download.h
 *
 * Copyright © 2017 Julian Richen <julian@richen.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SLURP_DOWNLOAD_H
#define SLURP_DOWNLOAD_H

#include "slurp-resource.h"
#include "slurp-download-data.h"

#include <glib-object.h>
#include <glib.h>
#include <gio/gio.h>
#include <libsoup/soup.h>

G_BEGIN_DECLS

#define SLURP_TYPE_DOWNLOAD (slurp_download_get_type())
#define SLURP_DOWNLOAD_ERROR slurp_download_error_quark ()

G_DECLARE_FINAL_TYPE (SlurpDownload, slurp_download, SLURP, DOWNLOAD, GObject)

struct _SlurpDownload
{
    GObject parent_instance;
};

/**
 * SlurpDownloadError:
 * @SLURP_DOWNLOAD_ERROR_NO_SOUP_SESSION: No #SoupSession set in #SlurpDownload.
 * @SLURP_DOWNLOAD_ERROR_NO_SOUP_SESSION_REQUEST: No #SoupRequest set.
 * @SLURP_DOWNLOAD_ERROR_NO_INPUT_STREAM: No #GInputStream set for resource source.
 * @SLURP_DOWNLOAD_ERROR_NO_OUTPUT: No #GOutputStream set for resource target.
 *
 * An error code used with the %SLURP_DOWNLOAD_ERROR domain.
 */
typedef enum _SlurpDownloadError {
    SLURP_DOWNLOAD_ERROR_NO_SOUP_SESSION,
    SLURP_DOWNLOAD_ERROR_NO_SOUP_REQUEST,
    SLURP_DOWNLOAD_ERROR_NO_INPUT_STREAM,
    SLURP_DOWNLOAD_ERROR_NO_OUTPUT_STREAM
} SlurpDownloadError;

GQuark
slurp_download_error_quark (void);

SlurpDownload *
slurp_download_new (void);

SlurpDownload *
slurp_download_new_with_session (SoupSession *session);

void
slurp_download_resource_async (SlurpDownload       *self,
                               SlurpResource       *resource,
                               GCancellable        *cancellable,
                               GAsyncReadyCallback  callback,
                               gpointer             user_data);

gboolean
slurp_download_resource_finish (SlurpDownload  *self,
                                GAsyncResult   *result,
                                GError        **error);

G_END_DECLS

#endif /* SLURP_DOWNLOAD_H */

